//
//  amorim_xmlPlugIn.m
//  amorim_xml
//
//  Created by Dustin O'Connor on 10/29/10.
//  Copyright (c) 2010 d0cut0uch. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "amorim_xmlPlugIn.h"

#define	kQCPlugIn_Name				@"amorim_xml"
#define	kQCPlugIn_Description		@"amorim_xml description"

@implementation amorim_xmlPlugIn

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic inputPath, inputWrite, inputStructure;

+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	 normally you want to do this we are not cause it says optional. if we have problems then we will later.
	 */


	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeConsumer;
}

+ (QCPlugInTimeMode) timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeIdle; //doesnt matter put to idle this is for external time base..kQCPlugInTimeModeTimeBase when you right click patch set to external time no need for this plug..
}

- (id) init
{
	if(self = [super init]) {
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void) finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void) dealloc
{
	/*
	Release any resources created in -init.
	*/
	
	[super dealloc];
}

@end

@implementation amorim_xmlPlugIn (Execution)

- (BOOL) startExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition starts: perform any required setup for the plug-in.
	Return NO in case of fatal failure (this will prevent rendering of the composition to start).
	*/
	
	return YES;
}

- (void) enableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
	
    BOOL writer = self.inputWrite;
    if(writer) 
    {
        NSString * error;
		
		NSEnumerator *enumerator = [self.inputStructure keyEnumerator];
		id key;
		
		NSMutableArray *index = [[NSMutableArray alloc] init];
		
		NSArray *myKeys = [self.inputStructure allKeys];
		NSArray *sortedKeys = [myKeys sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
		
		int i = 0;
		while ((key = [enumerator nextObject])) {
			 {
				 [index addObject:[self.inputStructure objectForKey: [sortedKeys objectAtIndex:i]]];
				 
				 i++;
			}
		}
		
	
        id plist = [NSPropertyListSerialization dataFromPropertyList:(id)index
                                                              format:NSPropertyListXMLFormat_v1_0 errorDescription:&error];
		
		
		if(plist) {
			NSLog(@"No error creating XML data.");
			[plist writeToFile:self.inputPath atomically:YES];
		}
		else {
			NSLog(error);
			[error release];
		}
		
	
		
    }
	
	return YES;
}
		
		 
	- (void) disableExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when rendering of the composition stops: perform any required cleanup for the plug-in.
	*/
}

@end
